import logging
import subprocess
import json
import urllib
import urllib.request
import shlex
import jetpack.util
import jetpack.nodehealthchecks.report_error
import jetpack.status
from jetpack.status import JetpackStatus, JetpackResource

NODEHEALTHCHECKS_TEST_WRAPPER = "/opt/cycle/jetpack/system/scripts/run-tests.sh"
AZUREHPC_TESTS_PATH = "/opt/azurehpc/test/azurehpc-health-checks/run-health-checks.sh"
NODEHEALTHCHECKS_LOG_PATH = "/opt/cycle/jetpack/logs"
PATH_TO_COMPONENT_VERSIONS = "/opt/azurehpc/component_versions.txt"
HEALTHCHECKS_COMPONENT_NAME = "AZ_HEALTH_CHECKS"
JETPACK_PYTHON_PATH = "/opt/cycle/jetpack/system/python/bin/python3"


class NodehealthcheckException(RuntimeError):
    pass


logger = logging.getLogger(__name__)


def fetch_instance_metadata():
    header_url = "http://169.254.169.254/metadata/instance/compute?api-version=2021-02-01"
    opener = urllib.request.build_opener()
    opener.addheaders = [('Metadata', 'true')]
    urllib.request.install_opener(opener)
    response = urllib.request.urlopen(header_url)
    data = response.read()
    parsed_data = json.loads(data)
    return parsed_data


def run_test(path, test_arguments=None, bash_wrapper=NODEHEALTHCHECKS_TEST_WRAPPER, 
             log_path=NODEHEALTHCHECKS_LOG_PATH, component_version_path=PATH_TO_COMPONENT_VERSIONS, 
             healthchecks_component_name=HEALTHCHECKS_COMPONENT_NAME, python_path=JETPACK_PYTHON_PATH):
    test_results = None
    # we always run tests using a special bash wrapper
    command = [bash_wrapper, path, log_path, component_version_path, healthchecks_component_name, python_path]
    if test_arguments:
        # allow test_arguments passed as a single string
        if isinstance(test_arguments, str):
            test_arguments = shlex.split(test_arguments)
        command.extend(test_arguments)
    proc = subprocess.Popen(command, stdout=subprocess.PIPE)
    stdout, _stderr = proc.communicate()
    # we normally expect a valid json output from our own wrapper script
    test_results = json.loads(stdout, strict=False)
    return test_results


def healthcheck(test_path=None, test_arguments=None, fail_test=False, cluster_session_id=None):
    logger.info("Running nodehealthecks...")
    vm_metadata = fetch_instance_metadata()
    
    if jetpack.util.is_windows():
        logger.warning("Windows is not supported by NodeHealthChecks.")
        test_results = {
            'exit_code': 0,
            'message': "Windows is not supported by NodeHealthChecks.",
            'test_path': "/this/is/fakehealthcheck",
            'vm_metadata': vm_metadata
        }
        jetpack.nodehealthchecks.report_error.report_error(test_results, level="warn", cluster_session_id=cluster_session_id)
        return

    if test_path is None:
        test_path = AZUREHPC_TESTS_PATH
    if fail_test:
        test_results = {
            'exit_code': 1,
            'message': "Fake failure",
            'test_path': "/this/is/fakehealthcheck"
        }
    else:
        test_results = run_test(test_path, test_arguments)
    
    if test_results['exit_code'] != 0:
        test_results['vm_metadata'] = vm_metadata
        error_message = f"NodeHealthChecks test failed: {test_results['test_path']}"
        logger.error(error_message)
        jetpack.nodehealthchecks.report_error.report_error(test_results, cluster_session_id=cluster_session_id)
        raise NodehealthcheckException(error_message)
    # when healthchecks are not supported, we want them to pass with a warning
    if test_results['message'] != "HealthChecks passed":
        test_results['vm_metadata'] = vm_metadata
        logger.warning(test_results['message'])
        healthchecks_status = JetpackStatus.Warning
        healthchecks_message = "NodeHealthChecks not supported"
    else:
        healthchecks_status = JetpackStatus.Ok
        logger.info(test_results['message'])
        healthchecks_message = "NodeHealthChecks ran successfully"
        
    jetpack_status = {
        'name': JetpackResource.NodeHealthChecks,
        'status': healthchecks_status,
        'message': healthchecks_message,
        'description': test_results['message'],
    }
    
    jetpack.status.update(**jetpack_status, update_only=True)
